@@ -16,12 +16,12 @@ pub const LABEL_ATTR: &str = "LBL";
 lazy_static! {
     pub static ref TYPE_INVARIANT: InvariantEntry = InvariantEntry {
         attribute: String::from(TYPE_BASE_ATTR),
-        value: EntryValue::Value(serde_json::Value::from(TYPE_TYPE_VAL)),
+        value: EntryValue::String(String::from(TYPE_TYPE_VAL)),
     };
     pub static ref TYPE_ADDR: Address = TYPE_INVARIANT.entity().unwrap();
     pub static ref HIER_INVARIANT: InvariantEntry = InvariantEntry {
         attribute: String::from(TYPE_BASE_ATTR),
-        value: EntryValue::Value(serde_json::Value::from(HIER_TYPE_VAL)),
+        value: EntryValue::String(String::from(HIER_TYPE_VAL)),
     };
     pub static ref HIER_ADDR: Address = HIER_INVARIANT.entity().unwrap();
 }
