@@ -1,6 +1,6 @@
 use crate::addressing::{Address, Addressable};
 use crate::database::inner::models;
-use crate::util::hash::{decode, hash, Hash, Hashable};
+use crate::util::hash::{b58_decode, hash, Hash, Hashable};
 use anyhow::{anyhow, Result};
 use serde::{Deserialize, Serialize};
 use std::convert::TryFrom;
@@ -32,7 +32,8 @@ pub struct InvariantEntry {
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(tag = "t", content = "c")]
 pub enum EntryValue {
-    Value(serde_json::Value),
+    String(String),
+    Number(f64),
     Address(Address),
     Invalid,
 }
@@ -41,11 +42,23 @@ impl TryFrom<&models::Entry> for Entry {
     type Error = anyhow::Error;
 
     fn try_from(e: &models::Entry) -> Result<Self, Self::Error> {
-        Ok(Entry {
-            entity: Address::decode(&e.entity)?,
-            attribute: e.attribute.clone(),
-            value: e.value.parse().unwrap(),
-        })
+        if let Some(value_str) = &e.value_str {
+            Ok(Entry {
+                entity: Address::decode(&e.entity)?,
+                attribute: e.attribute.clone(),
+                value: value_str.parse()?,
+            })
+        } else if let Some(value_num) = e.value_num {
+            Ok(Entry {
+                entity: Address::decode(&e.entity)?,
+                attribute: e.attribute.clone(),
+                value: EntryValue::Number(value_num),
+            })
+        } else {
+            Err(anyhow!(
+                "Inconsistent database: Both values of entry are NULL!"
+            ))
+        }
     }
 }
 
@@ -53,13 +66,24 @@ impl TryFrom<&Entry> for models::Entry {
     type Error = anyhow::Error;
 
     fn try_from(e: &Entry) -> Result<Self, Self::Error> {
-        Ok(models::Entry {
-            identity: e.address()?.encode()?,
-            entity: e.entity.encode()?,
-            attribute: e.attribute.clone(),
-            value: e.value.to_string()?,
-            immutable: false,
-        })
+        match e.value {
+            EntryValue::Number(n) => Ok(models::Entry {
+                identity: e.address()?.encode()?,
+                entity: e.entity.encode()?,
+                attribute: e.attribute.clone(),
+                value_str: None,
+                value_num: Some(n),
+                immutable: false,
+            }),
+            _ => Ok(models::Entry {
+                identity: e.address()?.encode()?,
+                entity: e.entity.encode()?,
+                attribute: e.attribute.clone(),
+                value_str: Some(e.value.to_string()?),
+                value_num: None,
+                immutable: false,
+            }),
+        }
     }
 }
 
@@ -67,13 +91,24 @@ impl TryFrom<&ImmutableEntry> for models::Entry {
     type Error = anyhow::Error;
 
     fn try_from(e: &ImmutableEntry) -> Result<Self, Self::Error> {
-        Ok(models::Entry {
-            identity: e.0.address()?.encode()?,
-            entity: e.0.entity.encode()?,
-            attribute: e.0.attribute.clone(),
-            value: e.0.value.to_string()?,
-            immutable: true,
-        })
+        match e.0.value {
+            EntryValue::Number(n) => Ok(models::Entry {
+                identity: e.0.address()?.encode()?,
+                entity: e.0.entity.encode()?,
+                attribute: e.0.attribute.clone(),
+                value_str: None,
+                value_num: Some(n),
+                immutable: false,
+            }),
+            _ => Ok(models::Entry {
+                identity: e.0.address()?.encode()?,
+                entity: e.0.entity.encode()?,
+                attribute: e.0.attribute.clone(),
+                value_str: Some(e.0.value.to_string()?),
+                value_num: None,
+                immutable: false,
+            }),
+        }
     }
 }
 
@@ -144,7 +179,8 @@ impl Addressable for InvariantEntry {}
 impl EntryValue {
     pub fn to_string(&self) -> Result<String> {
         let (type_char, content) = match self {
-            EntryValue::Value(value) => ('J', serde_json::to_string(value)?),
+            EntryValue::String(value) => ('S', value.to_owned()),
+            EntryValue::Number(n) => ('N', n.to_string()),
             EntryValue::Address(address) => ('O', address.to_string()),
             EntryValue::Invalid => return Err(anyhow!("Cannot serialize invalid Entity value.")),
         };
@@ -157,10 +193,8 @@ impl std::fmt::Display for EntryValue {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let (entry_type, entry_value) = match self {
             EntryValue::Address(address) => ("ADDRESS", address.to_string()),
-            EntryValue::Value(value) => (
-                "VALUE",
-                serde_json::to_string(value).unwrap_or_else(|_| String::from("?!?!?!")),
-            ),
+            EntryValue::String(string) => ("STRING", string.to_owned()),
+            EntryValue::Number(n) => ("NUMBER", n.to_string()),
             EntryValue::Invalid => ("INVALID", "INVALID".to_string()),
         };
         write!(f, "{}: {}", entry_type, entry_value)
@@ -176,15 +210,16 @@ impl std::str::FromStr for EntryValue {
         } else {
             let (type_char, content) = s.split_at(1);
             match (type_char, content) {
-                ("J", content) => {
-                    if let Ok(value) = serde_json::from_str(content) {
-                        Ok(EntryValue::Value(value))
+                ("S", content) => Ok(EntryValue::String(String::from(content))),
+                ("N", content) => {
+                    if let Ok(n) = content.parse::<f64>() {
+                        Ok(EntryValue::Number(n))
                     } else {
                         Ok(EntryValue::Invalid)
                     }
                 }
                 ("O", content) => {
-                    if let Ok(addr) = decode(content).and_then(|v| Address::decode(&v)) {
+                    if let Ok(addr) = b58_decode(content).and_then(|v| Address::decode(&v)) {
                         Ok(EntryValue::Address(addr))
                     } else {
                         Ok(EntryValue::Invalid)
@@ -195,3 +230,29 @@ impl std::str::FromStr for EntryValue {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use anyhow::Result;
+
+    #[test]
+    fn test_value_from_to_string() -> Result<()> {
+        let entry = EntryValue::String("hello".to_string());
+        let encoded = entry.to_string()?;
+        let decoded = encoded.parse::<EntryValue>()?;
+        assert_eq!(entry, decoded);
+
+        let entry = EntryValue::Number(1337.93);
+        let encoded = entry.to_string()?;
+        let decoded = encoded.parse::<EntryValue>()?;
+        assert_eq!(entry, decoded);
+
+        let entry = EntryValue::Address(Address::Url("https://upendproject.net".to_string()));
+        let encoded = entry.to_string()?;
+        let decoded = encoded.parse::<EntryValue>()?;
+        assert_eq!(entry, decoded);
+
+        Ok(())
+    }
+}
