@@ -82,14 +82,8 @@ export class UpObject {
   }
 
   public identify(): string[] {
-    const hasAliases = this.backlinks
-      .filter((entry) => entry.attribute === "HAS")
-      .map((entry) => entry.get("ALIAS"))
-      .filter(Boolean) as string[];
-
     const lblValues = (this.attr["LBL"] || []).map((e) => String(e.value.c));
-
-    return lblValues.concat(hasAliases);
+    return lblValues;
   }
 
   public asDict() {
