@@ -1,6 +1,6 @@
 import type { UpEntry } from "upend";
 import type { ListingResult } from "upend/types";
-import { fetchEntry, query as queryFn, queryOnce } from "../lib/entity";
+import { query as queryFn, queryOnce } from "../lib/entity";
 
 export function baseSearch(query: string) {
   return queryFn(
@@ -19,21 +19,10 @@ export async function getObjects(
   entries: UpEntry[],
   limit = 50
 ): Promise<[string, string][]> {
-  const labelled = entries
+  return entries
     .slice(0, limit)
     .filter((e) => e.attribute == "LBL")
     .map((e) => [e.entity, String(e.value.c)] as [string, string]);
-  const aliased = entries
-    .slice(0, limit)
-    .filter((e) => e.attribute === "ALIAS")
-    .map(async (aliasEntry) => {
-      const entry = await fetchEntry(aliasEntry.entity);
-      return [String(entry.value.c), String(aliasEntry.value.c)] as [
-        string,
-        string
-      ];
-    });
-  return labelled.concat(await Promise.all(aliased));
 }
 
 export async function createLabelled(label: string) {
